@@ -459,6 +459,7 @@ void PacketReader::getLabelFromContent(const vector<uint8_t>& content, uint16_t&
     throw MOADNSException("Loop");
 
   int pos = frompos;
+  // it is tempting to call reserve on ret, but it turns out it creates a malloc/free storm in the loop
   for(;;) {
     unsigned char labellen=content.at(frompos++);
 
@@ -479,7 +480,7 @@ void PacketReader::getLabelFromContent(const vector<uint8_t>& content, uint16_t&
       throw MOADNSException("Overly long label during label decompression ("+lexical_cast<string>((unsigned int)labellen)+")");
     else {
       // XXX FIXME THIS MIGHT BE VERY SLOW!
-      ret.reserve(ret.size() + labellen + 2);
+
       for(string::size_type n = 0 ; n < labellen; ++n, frompos++) {
         if(content.at(frompos)=='.' || content.at(frompos)=='\\') {
           ret.append(1, '\\');
@@ -493,6 +494,8 @@ void PacketReader::getLabelFromContent(const vector<uint8_t>& content, uint16_t&
       }
       ret.append(1,'.');
     }
+    if (ret.length() > 1024)
+      throw MOADNSException("Total name too long");
   }
 }
 
